import click

from .converter import Converter


FILE_PATH = ".coverage"
RELATIVE_BOOL = True
CONFIG_FILE = True  # use True as default value


@click.command()
@click.option("--data_file_path", default=".coverage", help="Path to .coverage file")
@click.option(
    "--output_file_path", default="lcov.info", help="lcov.info output file path"
)
@click.option("--config_file", default=True, help="Path to .coveragerc file")
@click.option("--relative_path", is_flag=True, help="Use relative path in LCOV output")
@click.option("--preview", is_flag=True, help="Preview LCOV output")
def main(data_file_path, output_file_path, config_file, relative_path, preview):
    print(relative_path)
    converter = Converter(
        data_file_path=data_file_path,
        config_file=config_file,
        relative_path=relative_path,
    )
    if preview:
        converter.print_lcov()
    else:
        converter.create_lcov(output_file_path)



import logging
from typing import Union

import coverage
from coverage.files import FnmatchMatcher, prep_patterns
from coverage.misc import CoverageException, NoSource, NotPython

log = logging.getLogger("coveragepy_lcov.converter")


class Converter:
    def __init__(
        self,
        relative_path: bool,
        config_file: Union[str, bool],
        data_file_path: str,
    ):
        self.relative_path = relative_path

        self.cov_obj = coverage.coverage(
            data_file=data_file_path, config_file=config_file
        )
        self.cov_obj.load()
        self.cov_obj.get_data()

    def get_lcov(self) -> str:
        """Get LCOV output

        This is shamelessly adapted from https://github.com/nedbat/coveragepy/blob/master/coverage/report.py

        """
        output = ""

        file_reporters = self.cov_obj._get_file_reporters(None)
        config = self.cov_obj.config

        if config.report_include:
            matcher = FnmatchMatcher(
                prep_patterns(config.report_include), "report_include"
            )
            file_reporters = [fr for fr in file_reporters if matcher.match(fr.filename)]

        if config.report_omit:
            matcher = FnmatchMatcher(prep_patterns(config.report_omit), "report_omit")
            file_reporters = [
                fr for fr in file_reporters if not matcher.match(fr.filename)
            ]

        if not file_reporters:
            raise CoverageException("No data to report.")

        for fr in sorted(file_reporters):
            try:
                analysis = self.cov_obj._analyze(fr)
                token_lines = analysis.file_reporter.source_token_lines()
                if self.relative_path:
                    filename = fr.relative_filename()
                else:
                    filename = fr.filename
                output += "TN:\n"
                output += f"SF:{filename}\n"

                lines_hit = 0
                for i, _ in enumerate(token_lines, 1):
                    hits = get_hits(i, analysis)
                    if hits is not None:
                        if hits > 0:
                            lines_hit += 1
                        output += f"DA:{i},{hits}\n"

                output += f"LF:{len(analysis.statements)}\n"
                output += f"LH:{lines_hit}\n"
                output += "end_of_record\n"

            except NoSource:
                if not config.ignore_errors:
                    raise

            except NotPython:
                if fr.should_be_python():
                    if config.ignore_errors:
                        msg = "Couldn't parse Python file '{}'".format(fr.filename)
                        self.cov_obj._warn(msg, slug="couldnt-parse")

                    else:
                        raise

        return output

    def print_lcov(self) -> None:
        """Print LCOV output

        Print out the LCOV output

        """
        lcov_str = self.get_lcov()
        print(lcov_str)

    def create_lcov(self, output_file_path) -> None:
        lcov_str = self.get_lcov()

        with open(output_file_path, "w") as output_file:
            output_file.write(lcov_str)


def get_hits(line_num, analysis):
    if line_num in analysis.missing:
        return 0

    if line_num not in analysis.statements:
        return None

    return 1



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['coveragepy_lcov']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0', 'coverage>=5.5,<6.0']

entry_points = \
{'console_scripts': ['coveragepy-lcov = coveragepy_lcov.cli:main']}

setup_kwargs = {
    'name': 'coveragepy-lcov',
    'version': '0.1.1',
    'description': 'A simple .coverage to LCOV converter',
    'long_description': '# coveragepy-lcov\n\nThis package provides a simple CLI for converting .coverage files to the LCOV format.\n\n# Usage\n\n```bash\npip install coveragepy-lcov\n\n# If the .coverage file is in your current working directory\ncoveragepy-lcov\n\n# Point to a different .coverage file path\ncoveragepy-lcov --data_file_path example/.coverage\n\n# Write the output to a different file path\ncoveragepy-lcov --output_file_path build/lcov.info\n\n# Use relative paths in the LCOV output\ncoveragepy-lcov --relative_path\n```\n\n# Configuration\n\n```text\nUsage: coveragepy-lcov [OPTIONS]\n\nOptions:\n  --data_file_path TEXT    Path to .coverage file\n  --output_file_path TEXT  lcov.info output file path\n  --config_file TEXT       Path to .coveragerc file\n  --relative_path          Use relative path in LCOV output\n  --preview                Preview LCOV output\n  --help                   Show this message and exit.\n```\n',
    'author': 'Chay Choong',
    'author_email': 'chaychoong@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/chaychoong/coveragepy-lcov',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
